package com.albertoventurini.parsley.grammar;

import javax.annotation.Nonnull;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

public abstract class ParseTree {

    private final String name;

    private final String text;

    private final List<ParseTree> children;

    public static final class Leaf extends ParseTree {
        public Leaf(
                @Nonnull final String name,
                @Nonnull final String text) {
            super(name, text, Collections.emptyList());
        }
    }

    public static final class Node extends ParseTree {
        public Node(
                @Nonnull final String name,
                @Nonnull final String text,
                @Nonnull final List<ParseTree> children) {
            super(name, text, children);
        }
    }

    public ParseTree(
            @Nonnull final String name,
            @Nonnull final String text,
            @Nonnull final List<ParseTree> children) {
        this.name = name;
        this.text = text;
        this.children = children;
    }

    public static Leaf leaf(
            @Nonnull final String name,
            @Nonnull final String text) {
        return new Leaf(name, text);
    }

    public static Node node(
            @Nonnull final String name,
            @Nonnull final String text,
            @Nonnull final List<ParseTree> children) {
        return new Node(name, text, children);
    }

    public String getText() {
        return text;
    }

    public List<ParseTree> getChildren() {
        return children;
    }

    public ParseTree getChild(final int i) {
        return children.get(i);
    }

    public Optional<ParseTree> getFirstDescendantByName(@Nonnull final String name) {
        if (name.equals(this.name)) {
            return Optional.of(this);
        }

        for (final ParseTree child : children) {
            final Optional<ParseTree> descendant = child.getFirstDescendantByName(name);
            if (descendant.isPresent()) {
                return descendant;
            }
        }

        return Optional.empty();
    }

    public ParseTree getChild(@Nonnull final String name) {
        return children.stream().filter(c -> name.equals(c.text))
                .findFirst()
                .orElseThrow(() -> new RuntimeException("Child not found: " + name));
    }

    @Override
    public String toString() {
        return "ParseTree{" +
                "name='" + name + '\'' +
                ", text='" + text + '\'' +
                '}';
    }
}
